/*
 * Filesystem client functions.
 *
 * Copyright (C) 2018, 2019, 2020, 2021, 2022,
 *               2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <dirent.h>
#include <fsclient/file.h>



EXTERN_C_BEGIN

/* Filesystem access operations. */

l4_cap_idx_t client_open_for_user(user_t user);
l4_cap_idx_t client_open_for_user_using(user_t user, l4_cap_idx_t server);

/* Opening and closing operations. */

void client_close(file_t *file);
file_t *client_open(const char *name, flags_t flags);
file_t *client_open_using(const char *name, flags_t flags, l4_cap_idx_t server);

file_t *client_opendir(const char *name);
file_t *client_opendir_using(const char *name, l4_cap_idx_t server);

file_t *client_opendir_at(file_t *file);
file_t *client_reopen(file_t *file, flags_t flags);

long client_pipe(file_t **reader, file_t **writer, flags_t flags);
long client_pipe_using(file_t **reader, file_t **writer, flags_t flags, l4_cap_idx_t server);

/* File status testing. */

int client_opened(file_t *file);

/* Other file operations. */

long client_mkdir(const char *path, mode_t mode);
long client_mkdir_using(const char *path, mode_t mode, l4_cap_idx_t server);

long client_remove(const char *path);
long client_remove_using(const char *path, l4_cap_idx_t server);

long client_rename(const char *source, const char *target);
long client_rename_using(const char *source, const char *target, l4_cap_idx_t server);

long client_stat(const char *path, struct stat *st);
long client_stat_using(const char *path, struct stat *st, l4_cap_idx_t server);

/* File and region operations. */

long client_flush(file_t *file);
void *client_mmap(file_t *file, offset_t position, offset_t length,
                  offset_t start_visible, offset_t end_visible,
                  l4re_rm_flags_t region_flags);
l4re_rm_flags_t client_region_flags(prot_t prot, flags_t flags);

/* Pipe region operations. */

long client_current_region(file_t *file);
long client_next_region(file_t *file);

/* File data operations. */

offset_t client_read(file_t *file, void *buf, offset_t count);
offset_t client_write(file_t *file, const void *buf, offset_t count);

/* File navigation operations. */

offset_t client_seek(file_t *file, offset_t offset, int whence);
offset_t client_tell(file_t *file);

/* Directory reading operations. */

struct dirent *client_readdir(file_t *file);

/* Notification operations. */

long client_set_blocking(file_t *file, notify_flags_t flags);

/* More advanced notification operations. */

void client_notifier_close(notifier_t *notifier);
notifier_t *client_notifier_local(void);
notifier_t *client_notifier_task(void);

long client_subscribe(file_t *file, notify_flags_t flags, notifier_t *notifier);
long client_unsubscribe(file_t *file, notifier_t *notifier);

long client_wait_file(file_t *file, notifier_t *notifier);
long client_wait_files(file_t **file, notifier_t *notifier);

EXTERN_C_END

// vim: tabstop=2 expandtab shiftwidth=2
